@@ -1,6 +1,7 @@
 # Mail::Header.pm
 #
-# Copyright (c) 1995-7 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 1995-2001 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 2002-2003 Mark Overmeer <mailtools@overmeer.net>
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
@@ -19,7 +20,7 @@ use strict;
 use Carp;
 use vars qw($VERSION $FIELD_NAME);
 
-$VERSION = "1.19";
+$VERSION = "1.67";
 
 my $MAIL_FROM = 'KEEP';
 my %HDR_LENGTHS = ();
@@ -86,7 +87,7 @@ my %STRUCTURE;
   Resent-Date Resent-From Resent-Sender Resent-To Return-Path
   list-help list-post list-unsubscribe Mailing-List
   Received References Message-ID In-Reply-To
-  Content-Length Content-Type
+  Content-Length Content-Type Content-Disposition
   Delivered-To
   Lines
   MIME-Version
@@ -105,14 +106,14 @@ sub _fold_line
  my $min = int($maxlen * 4 / 5) - 4;
  my $ml = $maxlen;
 
- $_[0] =~ s/\s*[\r\n]+\s*/ /og; # Compress any white space around a newline
+ $_[0] =~ s/[\r\n]+/ /og;       # Remove new-lines
  $_[0] =~ s/\s*\Z/\n/so;        # End line with a EOLN
 
  return if $_[0] =~ /^From\s/io;
 
  if(length($_[0]) > $ml)
   {
-   if ($_[0] =~ /^([-\w]+)/ and exists $STRUCTURE{ lc $1 } )
+   if ($_[0] =~ /^([-\w]+)/ && exists $STRUCTURE{ lc $1 } )
     {
      #Split the line up
      # first bias towards splitting at a , or a ; >4/5 along the line
@@ -120,11 +121,11 @@ sub _fold_line
      # else we are looking at a single word and probably don't want to split
      my $x = "";
 
-     $x .= "$1\n    "
-	while($_[0] =~ s/^\s*(
-			   [^"]{$min,$max}?[\,\;]
+     $x .= "$1\n "
+	while($_[0] =~ s/^\s*
+                          ([^"]{$min,$max}?[\,\;]
 			  |[^"]{1,$max}\s
-			  |[^\s"]*(?:"[^"]*"[^\s"]*)+\s
+			  |[^\s"]*(?:"[^"]*"[ \t]?[^\s"]*)+\s
 			  |[^\s"]+\s
 			  )
 			//x);
@@ -135,14 +136,8 @@ sub _fold_line
     }
    else
     {
-      my $dif = $max-$min;
-
-      $_[0] =~ s/(?:^|\G)
-		(?:
-		  (.{$min,$max})\s+
-		 |(.{$min,$max})
-		)
-                /$+\n    /xg;
+      $_[0] =~ s/(.{$min,$max})\s+/$+\n /g;
+      $_[0] =~ s/\s*$/\n/s;
     }
   }
 
@@ -157,16 +152,12 @@ sub _fold_line
 sub _tag_case
 {
  my $tag = shift;
+ $tag =~ s/\:$//;
 
- $tag =~ s/:\Z//o;
-
- # Change the case of the tag
- # eq Message-Id
- $tag =~ s/\b([a-z]+)/\L\u$1/gio;
- $tag =~ s/\b([b-df-hj-np-tv-z]+|MIME)\b/\U$1/gio
-	if $tag =~ /-/;
-
- $tag;
+ join('-',
+     map { /^[b-df-hj-np-tv-z]+$|^(?:MIME|SWE|SOAP|LDAP|ID)$/i
+         ? uc($_) : ucfirst(lc($_)) }
+              split(m/\-/, $tag, -1));
 }
 
 # format a complete line
@@ -242,24 +233,24 @@ sub _insert
 
  if($where < 0)
   {
-   $where = scalar(@{$me->{'mail_hdr_list'}}) + $where + 1;
+   $where = @{$me->{'mail_hdr_list'}} + $where + 1;
 
    $where = 0
 	if($where < 0);
   }
- elsif($where >= scalar(@{$me->{'mail_hdr_list'}}))
+ elsif($where >= @{$me->{'mail_hdr_list'}})
   {
-   $where = scalar(@{$me->{'mail_hdr_list'}});
+   $where = @{$me->{'mail_hdr_list'}};
   }
 
- my $atend = $where == scalar(@{$me->{'mail_hdr_list'}});
+ my $atend = $where == @{$me->{'mail_hdr_list'}};
 
  splice(@{$me->{'mail_hdr_list'}},$where,0,$line);
 
  $me->{'mail_hdr_hash'}{$tag} ||= [];
  my $ref = \${$me->{'mail_hdr_list'}}[$where];
 
- if(scalar($me->{'mail_hdr_hash'}{$tag}) && $where)
+ if($me->{'mail_hdr_hash'}{$tag} && $where)
   {
    if($atend)
     {
@@ -267,9 +258,8 @@ sub _insert
     }
    else
     {
-     my($ln,$i,$ref);
-     $i = 0;
-     foreach $ln (@{$me->{'mail_hdr_list'}})
+     my $i = 0;
+     foreach my $ln (@{$me->{'mail_hdr_list'}})
       {
        my $r = \$ln;
        last if($r == $ref);
@@ -393,8 +383,8 @@ sub unfold
    $list = $me->{'mail_hdr_hash'}{$tag};
    foreach $ln (@$list)
     {
-     $$ln =~ s/\r?\n\s+/ /sog
-	if defined $ln && defined $$ln;
+       $$ln =~ s/\r?\n\s+/ /sog
+           if defined $ln && defined $$ln;
     }
   }
  else
@@ -750,7 +740,7 @@ sub fold_length
    if(defined $len)
     {
      $me->{'mail_hdr_foldlen'} = $len > 20 ? $len : 20;
-     $me->fold;
+     $me->fold if $me->{'mail_hdr_modify'};
     }
   }
 
@@ -966,7 +956,7 @@ of tag will be removed.
 
 =item count ( TAG )
 
-Returns the number of times the given tag appears in the header
+Returns the number of times the given atg appears in the header
 
 =item print ( [ FD ] )
 
@@ -991,7 +981,7 @@ a description on how C<Mail::Header> uses these values.
 
 =item tags ()
 
-Returns an array of all the tags that exist in the header. Each tag will
+Retruns an array of all the tags that exist in the header. Each tag will
 only appear in the list once. The order of the tags is not specified.
 
 =item dup ()
@@ -1011,12 +1001,12 @@ multiple lines. IF C<TAG> is not given then all lines are unfolded.
 
 =head1 AUTHOR
 
-Graham Barr <gbarr@pobox.com>
+Graham Barr.  Maintained by Mark Overmeer <mailtools@overmeer.net>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1995-7 Graham Barr. All rights reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms
-as Perl itself.
+Copyright (c) 2002-2003 Mark Overmeer, 1995-2001 Graham Barr. All rights
+reserved. This program is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =cut
